@@ -90,6 +90,8 @@ gsasl_callback (Gsasl * ctx, Gsasl_session * sctx, Gsasl_property prop)
     Gsasl_server_callback_external cb_external;
     Gsasl_server_callback_securid cb_securid;
     Gsasl_server_callback_gssapi cb_gssapi;
+    Gsasl_server_callback_validate cb_validate;
+    Gsasl_server_callback_retrieve cb_retrieve;
     char buf[BUFSIZ];
     size_t buflen = BUFSIZ - 1;
     int res;
@@ -120,8 +122,11 @@ gsasl_callback (Gsasl * ctx, Gsasl_session * sctx, Gsasl_property prop)
 	  break;
 	res = cb_securid (sctx, sctx->authid, sctx->authzid, sctx->passcode,
 			  sctx->pin, buf, &buflen);
-	buf[buflen] = '\0';
-	gsasl_property_set (sctx, GSASL_SUGGESTED_PIN, buf);
+	if (buflen > 0 && buflen < BUFSIZ - 1)
+	  {
+	    buf[buflen] = '\0';
+	    gsasl_property_set (sctx, GSASL_SUGGESTED_PIN, buf);
+	  }
 	return res;
 	break;
 
@@ -133,6 +138,26 @@ gsasl_callback (Gsasl * ctx, Gsasl_session * sctx, Gsasl_property prop)
 	return res;
 	break;
 
+      case GSASL_VALIDATE_SIMPLE:
+	cb_validate = gsasl_server_callback_validate_get (sctx->ctx);
+	if (!cb_validate)
+	  break;
+	res = cb_validate (sctx, sctx->authzid, sctx->authid, sctx->password);
+	return res;
+	break;
+
+      case GSASL_PASSWORD:
+	cb_retrieve = gsasl_server_callback_retrieve_get (sctx->ctx);
+	if (!cb_retrieve)
+	  break;
+	res = cb_retrieve (sctx, sctx->authid, sctx->authzid,
+			   sctx->hostname, &buf, &buflen);
+	if (res == GSASL_OK)
+	  gsasl_property_set_raw (sctx, GSASL_PASSWORD, buf, buflen);
+	/* FIXME else if (res == GSASL_TOO_SMALL_BUFFER)... */
+	return res;
+	break;
+
       default:
 	break;
       }
