@@ -21,8 +21,6 @@
 
 #include "internal.h"
 
-#ifndef GSASL_NO_OBSOLETE
-
 /**
  * gsasl_client_listmech:
  * @ctx: libgsasl handle.
@@ -608,7 +606,7 @@ gsasl_appinfo_get (Gsasl_session * sctx)
 const char *
 gsasl_server_suggest_mechanism (Gsasl * ctx, const char *mechlist)
 {
-  return NULL;  /* This function is just silly. */
+  return NULL;			/* This function is just silly. */
 }
 
 /**
@@ -1541,4 +1539,465 @@ gsasl_server_callback_service_get (Gsasl * ctx)
 {
   return ctx ? ctx->cbs_service : NULL;
 }
+
+#if WITH_SASLPREP
+# include <stringprep.h>
 #endif
+
+/**
+ * gsasl_stringprep_nfkc:
+ * @in: a UTF-8 encoded string.
+ * @len: length of @str, in bytes, or -1 if @str is nul-terminated.
+ *
+ * Converts a string into canonical form, standardizing such issues as
+ * whether a character with an accent is represented as a base
+ * character and combining accent or as a single precomposed
+ * character.
+ *
+ * The normalization mode is NFKC (ALL COMPOSE).  It standardizes
+ * differences that do not affect the text content, such as the
+ * above-mentioned accent representation. It standardizes the
+ * "compatibility" characters in Unicode, such as SUPERSCRIPT THREE to
+ * the standard forms (in this case DIGIT THREE). Formatting
+ * information may be lost but for most text operations such
+ * characters should be considered the same. It returns a result with
+ * composed forms rather than a maximally decomposed form.
+ *
+ * Return value: Return a newly allocated string, that is the NFKC
+ *   normalized form of @str, o %NULL on error.
+ *
+ * Deprecated: No replacement functionality in GNU SASL, use GNU
+ * Libidn instead.  Note that in SASL, you most likely want to use
+ * SASLprep and not bare NFKC, see gsasl_saslprep().
+ **/
+char *
+gsasl_stringprep_nfkc (const char *in, ssize_t len)
+{
+  char *out = NULL;
+
+#if WITH_SASLPREP
+  out = stringprep_utf8_nfkc_normalize (in, len);
+#endif
+
+  return out;
+}
+
+/**
+ * gsasl_stringprep_saslprep:
+ * @in: input ASCII or UTF-8 string with data to prepare according to SASLprep.
+ * @stringprep_rc: pointer to output variable with stringprep error code,
+ *   or %NULL to indicate that you don't care about it.
+ *
+ * Process a Unicode string for comparison, according to the
+ * "SASLprep" stringprep profile.  This function is intended to be
+ * used by Simple Authentication and Security Layer (SASL) mechanisms
+ * (such as PLAIN, CRAM-MD5, and DIGEST-MD5) as well as other
+ * protocols exchanging user names and/or passwords.
+ *
+ * Return value: Return a newly allocated string that is the
+ *   "SASLprep" processed form of the input string, or %NULL on error,
+ *   in which case @stringprep_rc contain the stringprep library error
+ *   code.
+ *
+ * Deprecated: Use gsasl_saslprep() instead.
+ **/
+char *
+gsasl_stringprep_saslprep (const char *in, int *stringprep_rc)
+{
+  char *out = NULL;
+  int rc;
+
+#if WITH_SASLPREP
+  rc = stringprep_profile (in, &out, "SASLprep", 0);
+  if (stringprep_rc)
+    *stringprep_rc = rc;
+  if (rc != STRINGPREP_OK)
+    out = NULL;
+#endif
+
+  return out;
+}
+
+/**
+ * gsasl_stringprep_trace:
+ * @in: input ASCII or UTF-8 string with data to prepare according to "trace".
+ * @stringprep_rc: pointer to output variable with stringprep error code,
+ *   or %NULL to indicate that you don't care about it.
+ *
+ * Process a Unicode string for use as trace information, according to
+ * the "trace" stringprep profile.  The profile is designed for use
+ * with the SASL ANONYMOUS Mechanism.
+ *
+ * Return value: Return a newly allocated string that is the "trace"
+ *   processed form of the input string, or %NULL on error, in which
+ *   case @stringprep_rc contain the stringprep library error code.
+ *
+ * Deprecated: No replacement functionality in GNU SASL, use GNU
+ * Libidn instead.
+ **/
+char *
+gsasl_stringprep_trace (const char *in, int *stringprep_rc)
+{
+  char *out = NULL;
+  int rc;
+
+#if WITH_SASLPREP
+  rc = stringprep_profile (in, &out, "trace", 0);
+  if (stringprep_rc)
+    *stringprep_rc = rc;
+  if (rc != STRINGPREP_OK)
+    out = NULL;
+#endif
+
+  return out;
+}
+
+/**
+ * gsasl_md5pwd_get_password:
+ * @filename: filename of file containing passwords.
+ * @username: username string.
+ * @key: output character array.
+ * @keylen: input maximum size of output character array, on output
+ * contains actual length of output array.
+ *
+ * Retrieve password for user from specified file.  To find out how
+ * large the output array must be, call this function with out=NULL.
+ *
+ * The file should be on the UoW "MD5 Based Authentication" format,
+ * which means it is in text format with comments denoted by # first
+ * on the line, with user entries looking as username\tpassword.  This
+ * function removes \r and \n at the end of lines before processing.
+ *
+ * Return value: Return GSASL_OK if output buffer contains the
+ * password, GSASL_AUTHENTICATION_ERROR if the user could not be
+ * found, or other error code.
+ *
+ * Deprecated: Use gsasl_simple_getpass() instead.
+ **/
+int
+gsasl_md5pwd_get_password (const char *filename,
+			   const char *username, char *key, size_t * keylen)
+{
+  char matchbuf[BUFSIZ];
+  char line[BUFSIZ];
+  FILE *fh;
+
+  fh = fopen (filename, "r");
+  if (fh == NULL)
+    return GSASL_FOPEN_ERROR;
+
+  sprintf (matchbuf, "%s\t", username);
+
+  while (!feof (fh))
+    {
+      if (fgets (line, BUFSIZ, fh) == NULL)
+	break;
+
+      if (line[0] == '#')
+	continue;
+
+      while (strlen (line) > 0 && (line[strlen (line) - 1] == '\n' ||
+				   line[strlen (line) - 1] == '\r'))
+	line[strlen (line) - 1] = '\0';
+
+      if (strlen (line) <= strlen (matchbuf))
+	continue;
+
+      if (strncmp (matchbuf, line, strlen (matchbuf)) == 0)
+	{
+	  if (*keylen < strlen (line) - strlen (matchbuf))
+	    {
+	      fclose (fh);
+	      return GSASL_TOO_SMALL_BUFFER;
+	    }
+
+	  *keylen = strlen (line) - strlen (matchbuf);
+
+	  if (key)
+	    memcpy (key, &line[strlen (matchbuf)], *keylen);
+
+	  fclose (fh);
+
+	  return GSASL_OK;
+	}
+    }
+
+  if (fclose (fh) != 0)
+    return GSASL_FCLOSE_ERROR;
+
+  return GSASL_AUTHENTICATION_ERROR;
+}
+
+
+/*
+ * Copyright (c) 1996-1999 by Internet Software Consortium.
+ *
+ * Permission to use, copy, modify, and distribute this software for any
+ * purpose with or without fee is hereby granted, provided that the above
+ * copyright notice and this permission notice appear in all copies.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS" AND INTERNET SOFTWARE CONSORTIUM DISCLAIMS
+ * ALL WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES
+ * OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL INTERNET SOFTWARE
+ * CONSORTIUM BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL
+ * DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
+ * PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
+ * ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
+ * SOFTWARE.
+ */
+
+/*
+ * Portions Copyright (c) 1995 by International Business Machines, Inc.
+ *
+ * International Business Machines, Inc. (hereinafter called IBM) grants
+ * permission under its copyrights to use, copy, modify, and distribute this
+ * Software with or without fee, provided that the above copyright notice and
+ * all paragraphs of this notice appear in all copies, and that the name of IBM
+ * not be used in connection with the marketing of any product incorporating
+ * the Software or modifications thereof, without specific, written prior
+ * permission.
+ *
+ * To the extent it has a right to do so, IBM grants an immunity from suit
+ * under its patents, if any, for the use, sale or manufacture of products to
+ * the extent that such products are used for performing Domain Name System
+ * dynamic updates in TCP/IP networks by means of the Software.  No immunity is
+ * granted for any product per se or for any other function of any product.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", AND IBM DISCLAIMS ALL WARRANTIES,
+ * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
+ * PARTICULAR PURPOSE.  IN NO EVENT SHALL IBM BE LIABLE FOR ANY SPECIAL,
+ * DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER ARISING
+ * OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE, EVEN
+ * IF IBM IS APPRISED OF THE POSSIBILITY OF SUCH DAMAGES.
+ */
+
+#include <ctype.h>
+#include <stdio.h>
+
+#define Assert(Cond) if (!(Cond)) abort()
+
+static const char Base64[] =
+  "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
+static const char Pad64 = '=';
+
+/**
+ * gsasl_base64_encode:
+ * @src: input byte array
+ * @srclength: size of input byte array
+ * @target: output byte array
+ * @targsize: size of output byte array
+ *
+ * Encode data as base64.  Converts characters, three at a time,
+ * starting at src into four base64 characters in the target area
+ * until the entire input buffer is encoded.
+ *
+ * Return value: Returns the number of data bytes stored at the
+ * target, or -1 on error.
+ *
+ * Deprecated: Use gsasl_base64_to() instead.
+ **/
+int
+gsasl_base64_encode (char const *src,
+		     size_t srclength, char *target, size_t targsize)
+{
+  size_t datalength = 0;
+  unsigned char input[3];
+  unsigned char output[4];
+  size_t i;
+
+  while (2 < srclength)
+    {
+      input[0] = *src++;
+      input[1] = *src++;
+      input[2] = *src++;
+      srclength -= 3;
+
+      output[0] = input[0] >> 2;
+      output[1] = ((input[0] & 0x03) << 4) + (input[1] >> 4);
+      output[2] = ((input[1] & 0x0f) << 2) + (input[2] >> 6);
+      output[3] = input[2] & 0x3f;
+      Assert (output[0] < 64);
+      Assert (output[1] < 64);
+      Assert (output[2] < 64);
+      Assert (output[3] < 64);
+
+      if (datalength + 4 > targsize)
+	return (-1);
+      target[datalength++] = Base64[output[0]];
+      target[datalength++] = Base64[output[1]];
+      target[datalength++] = Base64[output[2]];
+      target[datalength++] = Base64[output[3]];
+    }
+
+  /* Now we worry about padding. */
+  if (0 != srclength)
+    {
+      /* Get what's left. */
+      input[0] = input[1] = input[2] = '\0';
+      for (i = 0; i < srclength; i++)
+	input[i] = *src++;
+
+      output[0] = input[0] >> 2;
+      output[1] = ((input[0] & 0x03) << 4) + (input[1] >> 4);
+      output[2] = ((input[1] & 0x0f) << 2) + (input[2] >> 6);
+      Assert (output[0] < 64);
+      Assert (output[1] < 64);
+      Assert (output[2] < 64);
+
+      if (datalength + 4 > targsize)
+	return (-1);
+      target[datalength++] = Base64[output[0]];
+      target[datalength++] = Base64[output[1]];
+      if (srclength == 1)
+	target[datalength++] = Pad64;
+      else
+	target[datalength++] = Base64[output[2]];
+      target[datalength++] = Pad64;
+    }
+  if (datalength >= targsize)
+    return (-1);
+  target[datalength] = '\0';	/* Returned value doesn't count \0. */
+  return (datalength);
+}
+
+/**
+ * gsasl_base64_decode:
+ * @src: input byte array
+ * @target: output byte array
+ * @targsize: size of output byte array
+ *
+ * Decode Base64 data.  Skips all whitespace anywhere.  Converts
+ * characters, four at a time, starting at (or after) src from Base64
+ * numbers into three 8 bit bytes in the target area.
+ *
+ * Return value: Returns the number of data bytes stored at the
+ * target, or -1 on error.
+ *
+ * Deprecated: Use gsasl_base64_from() instead.
+ **/
+int
+gsasl_base64_decode (char const *src, char *target, size_t targsize)
+{
+  int tarindex, state, ch;
+  char *pos;
+
+  state = 0;
+  tarindex = 0;
+
+  while ((ch = *src++) != '\0')
+    {
+      if (isspace (ch))		/* Skip whitespace anywhere. */
+	continue;
+
+      if (ch == Pad64)
+	break;
+
+      pos = strchr (Base64, ch);
+      if (pos == 0)		/* A non-base64 character. */
+	return (-1);
+
+      switch (state)
+	{
+	case 0:
+	  if (target)
+	    {
+	      if ((size_t) tarindex >= targsize)
+		return (-1);
+	      target[tarindex] = (pos - Base64) << 2;
+	    }
+	  state = 1;
+	  break;
+	case 1:
+	  if (target)
+	    {
+	      if ((size_t) tarindex + 1 >= targsize)
+		return (-1);
+	      target[tarindex] |= (pos - Base64) >> 4;
+	      target[tarindex + 1] = ((pos - Base64) & 0x0f) << 4;
+	    }
+	  tarindex++;
+	  state = 2;
+	  break;
+	case 2:
+	  if (target)
+	    {
+	      if ((size_t) tarindex + 1 >= targsize)
+		return (-1);
+	      target[tarindex] |= (pos - Base64) >> 2;
+	      target[tarindex + 1] = ((pos - Base64) & 0x03) << 6;
+	    }
+	  tarindex++;
+	  state = 3;
+	  break;
+	case 3:
+	  if (target)
+	    {
+	      if ((size_t) tarindex >= targsize)
+		return (-1);
+	      target[tarindex] |= (pos - Base64);
+	    }
+	  tarindex++;
+	  state = 0;
+	  break;
+	default:
+	  abort ();
+	}
+    }
+
+  /*
+   * We are done decoding Base-64 chars.  Let's see if we ended
+   * on a byte boundary, and/or with erroneous trailing characters.
+   */
+
+  if (ch == Pad64)
+    {				/* We got a pad char. */
+      ch = *src++;		/* Skip it, get next. */
+      switch (state)
+	{
+	case 0:		/* Invalid = in first position */
+	case 1:		/* Invalid = in second position */
+	  return (-1);
+
+	case 2:		/* Valid, means one byte of info */
+	  /* Skip any number of spaces. */
+	  for ((void) NULL; ch != '\0'; ch = *src++)
+	    if (!isspace (ch))
+	      break;
+	  /* Make sure there is another trailing = sign. */
+	  if (ch != Pad64)
+	    return (-1);
+	  ch = *src++;		/* Skip the = */
+	  /* Fall through to "single trailing =" case. */
+	  /* FALLTHROUGH */
+
+	case 3:		/* Valid, means two bytes of info */
+	  /*
+	   * We know this char is an =.  Is there anything but
+	   * whitespace after it?
+	   */
+	  for ((void) NULL; ch != '\0'; ch = *src++)
+	    if (!isspace (ch))
+	      return (-1);
+
+	  /*
+	   * Now make sure for cases 2 and 3 that the "extra"
+	   * bits that slopped past the last full byte were
+	   * zeros.  If we don't check them, they become a
+	   * subliminal channel.
+	   */
+	  if (target && target[tarindex] != 0)
+	    return (-1);
+	}
+    }
+  else
+    {
+      /*
+       * We ended by seeing the end of the string.  Make sure we
+       * have no partial bytes lying around.
+       */
+      if (state != 0)
+	return (-1);
+    }
+
+  return (tarindex);
+}
